@@ -2,11 +2,8 @@ from __future__ import annotations
 
 import argparse
 import configparser
-import copy
-import os
 import sys
 from abc import ABC, abstractmethod
-from ast import arg
 from configparser import ConfigParser
 from dataclasses import dataclass
 from enum import Enum
@@ -14,6 +11,7 @@ from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
     ClassVar,
     Dict,
     Generic,
@@ -31,8 +29,7 @@ from typing import (
 from typing_extensions import Annotated, get_args, get_origin, get_type_hints
 
 from .arg import Arg, Expander, Param, Positional
-from .errors import Err, Result, in_context
-from .types import ParamType
+from .errors import Err, Result, in_context, is_err, is_value
 from .util import ClassDoc, filter_types_single
 
 if TYPE_CHECKING:
@@ -216,7 +213,6 @@ class CLStdHandler(CLHandler):
         if handler is not None:
             next_args, err = handler.handle(args[1:], state)
             err = in_context(err, flag=flag)
-            # TODO: add context
             return next_args, err
         else:
             return self.fallback.handle(args, state)
@@ -394,7 +390,7 @@ class IniProcessor:
 
 
 @dataclass
-class ProcessorFactory:
+class ProcessorFactory(Generic[C]):
     """
     Describes a processor in construction
 
@@ -423,16 +419,18 @@ class ProcessorFactory:
     ini_section_strict: Dict[str, bool]
 
     #: List of handlers for key/value pairs present in INI files
-    ini_handlers: Dict[str, KVHandler]  # = {}
+    ini_handlers: Dict[str, KVHandler]
 
     #: List of command line flag handlers
-    cl_flag_handlers: Dict[str, CLHandler]  # = {}\
+    cl_flag_handlers: Dict[str, CLHandler]
 
     #: List of positional arguments
-    cl_positionals: List[Param[Any]]  # = []
+    cl_positionals: List[Param[Any]]
+
+    validators: List[Callable[[C], Optional[Err]]]
 
     @staticmethod
-    def make(config_type: Type[C]) -> ProcessorFactory:
+    def make(config_type: Type[C]) -> ProcessorFactory[C]:
         """
         Constructs an empty processor factory
 
@@ -470,6 +468,7 @@ class ProcessorFactory:
             ini_handlers={},
             cl_flag_handlers={},
             cl_positionals=[],
+            validators=[*config_type.validators_()],
         )
 
 
@@ -497,6 +496,8 @@ class Processor(Generic[C]):
     #: Dictionnary of parameters by field name
     params_by_name: Mapping[str, Param[Any]]
 
+    validators: Sequence[Callable[[C], Optional[Err]]]
+
     @staticmethod
     def process_fields(config_type: Type[C]) -> Sequence[Arg]:
         """
@@ -553,6 +554,7 @@ class Processor(Generic[C]):
             ini_processor=IniProcessor(pf.ini_section_strict, pf.ini_handlers),
             cl_handler=CLStdHandler(pf.cl_flag_handlers, CLPos(pf.cl_positionals)),
             params_by_name=pf.params_by_name,
+            validators=pf.validators,
         )
 
     def process_config(self, cwd: Path, state: State) -> Optional[Err]:
@@ -572,7 +574,7 @@ class Processor(Generic[C]):
         for p in paths:
             err = self.ini_processor.process(cwd / p, state)
             if err is not None:
-                errors.append(err.in_context(ini_file=p))  # TODO: add context
+                errors.append(err.in_context(ini_file=p))
         return Err.collect(*errors)
 
     def process(
@@ -600,7 +602,7 @@ class Processor(Generic[C]):
             if handler is not None:
                 err = handler.handle(value, state)
                 if err is not None:
-                    errors.append(err.in_context(environment_variable=key))  # TODO: add context
+                    errors.append(err.in_context(environment_variable=key))
             err = self.process_config(cwd, state)
             if err is not None:
                 errors.append(err.in_context(environment_variable=key))
@@ -623,10 +625,20 @@ class Processor(Generic[C]):
         for name, param in self.params_by_name.items():
             instances = state.instances[name]
             res = param.collector.collect(instances)
-            if isinstance(res, Err):
+            if is_value(res) and param.validator is not None:
+                res1 = param.validator(res)
+                if res1 is not None:
+                    res = res1
+            if is_err(res):
                 errors.append(res.in_context(param=name))
             else:
                 collected[name] = res
+
         if errors:
             return Err.collect_non_empty(*errors)
-        return self.config_type(**collected)
+        c: C = self.config_type(**collected)
+        validation_error: Optional[Err] = Err.collect_optional(*[f(c) for f in self.validators])
+        if validation_error is not None:
+            return validation_error
+        else:
+            return c
